require "active_support/core_ext/object/try"
require "active_support/core_ext/hash/slice"

module ActionDispatch::Routing
  # Extended ActionDispatch::Routing::Mapper implementation to add routing methods.
  class Mapper
    def generate_notifications_routes
      targetables = [
        { type: 'Hermes::Applicant', resources: :applicants },
        { type: 'Hermes::Entity', resources: :entities }
      ]

      targetables.each do |targetable|
        resources targetable[:resources] do
          member do
            resources :notifications, targetable_type: targetable[:type], controller: 'notification_center/notifications', only: %i[index show destroy], param: :notification_id do
              member do
                get :move
                put :open
              end
              collection do
                post :open_all
              end
            end
            resources :subscriptions, targetable_type: targetable[:type], controller: 'notification_center/subscriptions', only: %i[index show], param: :subscription_id do
              member do
                put :subscribe
                put :unsubscribe
              end
            end
          end
        end
      end
    end

    def generate_logs_routes
      resources :logs, only: %i[index show] do
        collection do
          ['HousingRequest', 'HousingOffer', 'Score', 'Applicant'].each do |logable|
            get logable.underscore.pluralize
          end
        end
      end
    end
  end
end