require 'rails_helper'

RSpec.describe Hermes::LogJob do
  subject { Hermes::LogJob }
  let(:redis_key) { 'redis_hermes_logs_queue' }

  before(:each) do
    $redis.del(redis_key)
  end

  it 'should set logs in queue' do
    perform_enqueued_jobs do
      subject.perform_now(code: '1234', action: 'test')
      queue = $redis.get redis_key
      expect(queue).not_to be_nil
    end
  end

  it 'should no create log if queue is not full' do
    perform_enqueued_jobs do
      5.times do
        subject.perform_now(code: '1234', action: 'test')
      end
      queue = $redis.get redis_key
      expect(queue).not_to be_nil

      queue = JSON.parse queue
      expect(queue).to be_an_instance_of(Array)
      expect(queue.count).to eq(5)
      expect(Log.count).to eq(0)
    end
  end

  it 'should create log if queue is full' do
    perform_enqueued_jobs do
      10.times do
        subject.perform_now(code: '1234', action: 'test')
      end
      
      expect(Log.count).to eq(10)
    end
  end

  it 'should create log if queue is full and keep other' do
    perform_enqueued_jobs do
      15.times do
        subject.perform_now(code: '1234', action: 'test')
      end
      
      expect(Log.count).to eq(10)
      queue = $redis.get redis_key
      expect(queue).not_to be_nil

      queue = JSON.parse queue
      expect(queue).to be_an_instance_of(Array)
      expect(queue.count).to eq(5)
    end
  end
end
